local name, ns = ...

local playerID
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
local GetContainerNumSlots = GetContainerNumSlots
local GetContainerItemLink = GetContainerItemLink
local GetContainerItemInfo = GetContainerItemInfo
local strsplit,format = strsplit,format
local resultStore = {}
ns['defaults'] = {
	['profile'] = {
		['icons'] = {},
		['names'] = {},
		['currencies'] = {},
		['bags'] = {},
		['bank'] = {},
		['units'] = {}
	}
}
ns['frame'] = CreateFrame("FRAME")
local function getPlayerID()
	if(playerID) then
		return playerID
	end
	local pname, pserver = UnitFullName("player")
	if(not pserver) then
		return
	end
	local player = format("%s - %s",pname,pserver)
	playerID = 0
	ns['db']['profile']['units'] = ns['db']['profile']['units'] or {}
	for k,v in pairs(ns['db']['profile']['units']) do
		if(v['name']==player) then
			playerID = k
			return playerID
		end
		if(k>=playerID) then
			playerID = k
		end
	end
	playerID = playerID + 1
	ns['db']['profile']['units'][playerID]= {
		['name'] = player,
		['faction'] = UnitFactionGroup('player')
	}
	return playerID
end
local function colorize(text,color)
	return string.format("|cff%s%s|r",color,text)
end
local function getCurrencyItem(index)
	local currencyLink = GetCurrencyListLink(index)
	if(currencyLink) then
		return currencyLink:match("currency:(%d+)") or nil
	end
end

local function updateCurrencies(playerID)
	if(playerID) then
		for index=1,GetCurrencyListSize() do
			local name, isHeader, isExpanded, isUnused, isWatched, count, icon, maximum, hasWeeklyLimit, currentWeeklyAmount =GetCurrencyListInfo(index)
			local itemID = getCurrencyItem(index)
			if((not isHeader) and itemID) then
				resultStore[itemID] = nil
				ns['db']['profile']['currencies'][playerID] = ns['db']['profile']['currencies'][playerID] or {}
				ns['db']['profile']['currencies'][playerID][itemID] = {
					['count'] = count,
					['max'] = maximum,
				}
				if(icon) then
					ns['db']['profile']['icons'][itemID] = "|T"..icon..":0|t"
				end
			end
		end
	end
end
ns['frame']['ADDON_LOADED'] = function(self,event,arg1)
	if(arg1 ~= name) then return end
	ns['db'] = LibStub("AceDB-3.0"):New(name .. "DB", ns['defaults'], true);
	self:UnregisterEvent("ADDON_LOADED")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")
	self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
	self:RegisterEvent("PLAYERREAGENTBANKSLOTS_CHANGED")
	self:RegisterEvent("GET_ITEM_INFO_RECEIVED")
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
end
ns['frame']:HookScript("OnEvent", function(self, event, ...)
	if(type(self[event])=='function') then
		self[event](self,event,...)
	end
 end)
ns['frame']:RegisterEvent("ADDON_LOADED")
local function getItemId(link)
	local id = nil
	if(link) then
		local _, _, itemString = string.find(link, "^|c%x+|H(.+)|h%[.*%]")
		_,id,_ = strsplit(":",itemString)
	end
	return id
end
local function bagLoop(res,id,cName,depot)
	local name = nil
	for k,v in pairs(ns['db']['profile'][depot]) do
		if(ns['db']['profile']['units'][k]) then
			if(ns['db']['profile']['units'][k]['faction']==ns['db']['profile']['units'][playerID]['faction']) then
				name = ns['db']['profile']['units'][k]['name']
				for bag,list in pairs(v) do
					for a,b in pairs(list) do
						if(a == id) then
							res[name] = res[name] or {
								[cName] = 0
							}
							res[name][cName] = res[name][cName] or 0
							res[name][cName] = res[name][cName] + b['count']
						end
					end
				end
			end
		end
	end
	return res
end
local function curLoop(res,id)
	local name = nil
	for k,v in pairs(ns['db']['profile']['currencies']) do
		if(ns['db']['profile']['units'][k]) then
			name = ns['db']['profile']['units'][k]['name']
			for a,b in pairs(v) do
				if(a == id and b['count']>0) then
					res[name] = b['count']
					if(b['max']>0) then
						res[name] = colorize(format("%s/%s",b['count'],b['max']),b['count']==b['max'] and "ff0000" or "ffffff")
					else
						res[name] = colorize(format("%s",b['count']),"ffffff")
					end
				end
			end
		end
	end
	return res
end
local function setName(id,itemname)
	if(id and itemname) then
		ns['db']['profile']['names'][id] = itemname
	end
end
local function getName(id)
	if(not ns['db']['profile']['names'][id]) then
		setName(id,GetItemInfo(id))
	end
	return ns['db']['profile']['names'][id]
end
local function getIcon(id)
	if(not ns['db']['profile']['icons'][id]) then
		local itemname, _, _, _, _, _, _, _, _, icon = GetItemInfo(id)
		setName(id,itemname)
		if(icon) then
			ns['db']['profile']['icons'][id] = "|T"..icon..":0|t"
		end
	end
	return ns['db']['profile']['icons'][id] or ''
end

local function getResult(id)
	if(resultStore[id]) then
		return resultStore[id]
	end
end
local function setResult(id,res)
	resultStore[id] = res
end

local function GameTooltip_OnTooltipSetItem(tooltip)
	local _, link = tooltip:GetItem()
	if(not link) then
		return
	end
	local id = getItemId(link)
	if(id)then
		tooltip:AddLine()
		local res = getResult(id)
		local isCurrency = link:match("currency:(%d+)")
		if(not res) then
			if(isCurrency) then
				res = curLoop({},isCurrency)
			else
				res = bagLoop({},id,'count','bags')
				res = bagLoop(res,id,'bankCount','bank')
			end
			setResult(id,res)
		end
		for name,b in pairs(res) do
			local dname, drealm = strsplit(" - ",name)
			if(isCurrency) then
				tooltip:AddLine(format("%s %s: %s",getIcon(id),dname,b))
			else
				local c1 = b['count'] and b['count'] or 0
				local c2 = b['bankCount'] and b['bankCount'] or 0
				local line,sep = '',''
				if(c1 > 0) then
					sep = c2>0 and ',' or ''
					line = format(" Bags %s%s",c1,sep)
				end
				if(c2 > 0) then
					line = format("%s Bank %s",line,c2)
				end
				tooltip:AddLine(format("%s %s: %s [%s ] ",getIcon(id),dname,c1 + c2, line))
			end
		end
	end
end

local function SetCurrencyByID(tooltip,itemID)
	if(itemID) then
		local res = getResult(itemID)
		if(not res) then
			res = curLoop({},itemID)
		end
		if(res) then
			tooltip:AddLine(" ")
			for name, b in pairs(res) do
				tooltip:AddLine(format("%s %s: %s",getIcon(itemID),name,b))
			end
		end
	end
	tooltip:Show()
	return tooltip,itemID
end
local function SetCurrencyToken(tooltip,index)
	SetCurrencyByID(tooltip,getCurrencyItem(index))
end
local function GameTooltip_SetHyperlink(tooltip, link)
	SetCurrencyByID(tooltip,link:match("currency:(%d+)"))
end
GameTooltip:HookScript("OnTooltipSetItem", GameTooltip_OnTooltipSetItem)
hooksecurefunc(GameTooltip,"SetHyperlink", GameTooltip_SetHyperlink)
hooksecurefunc(GameTooltip, "SetCurrencyByID", SetCurrencyByID)
hooksecurefunc(GameTooltip, "SetCurrencyToken", SetCurrencyToken)
local function scanBag(bag)
	local slot = 1
	local itemlist = {}
	local empty = true
	while(slot<=GetContainerNumSlots(bag)) do
		local texture, itemCount, locked, quality, readable, lootable, itemLink, isFiltered = GetContainerItemInfo(bag, slot)
		if(itemLink) then
			local id = getItemId(itemLink)
			if(id) then
				empty = false
				if(getResult(id)) then
					setResult(id,nil)
				end
				itemlist[id] = itemlist[id] or {}
				itemlist[id]['count'] = itemlist[id]['count'] or 0
				itemlist[id]['count'] = itemlist[id]['count'] + itemCount
				if(not ns['db']['profile']['names'][id]) then
					setName(id,GetItemInfo(id))
				end
			end
		end
		slot = slot + 1
	end
	return empty and nil or itemlist
end
local function scanAllBankBags()
	for bag=5,GetNumBankSlots()+4 do
		ns['db']['profile']['bank'][playerID][bag] = scanBag(bag) -- loop all bought bankbags
	end
	ns['db']['profile']['bank'][playerID][-1] = scanBag(-1) -- Main bankbag
	ns['db']['profile']['bank'][playerID][-3] = scanBag(-3) -- Main reagentbag
end
ns['frame']['GET_ITEM_INFO_RECEIVED'] = function(self, id)
	if(not ns['db']['profile']['names'][id]) then
		setName(id,GetItemInfo(id))
	end
end
ns['frame']['CURRENCY_DISPLAY_UPDATE'] = function()
	if(not playerID) then
		playerID = getPlayerID()
	end
	updateCurrencies(playerID)
end
ns['frame']['BANKFRAME_OPENED'] = function(self, event)
	ns['db']['profile']['bank'][playerID] = ns['db']['profile']['bank'][playerID] or {}
	scanAllBankBags()
end
ns['frame']['PLAYERBANKBAGSLOTS_CHANGED'] = ns['frame']['BANKFRAME_OPENED']
ns['frame']['PLAYERBANKSLOTS_CHANGED'] = ns['frame']['BANKFRAME_OPENED']
ns['frame']['PLAYERREAGENTBANKSLOTS_CHANGED'] = ns['frame']['BANKFRAME_OPENED']
ns['frame']['BAG_UPDATE'] = function(self, event, bag)
	if(not playerID) then
		playerID = getPlayerID()
	end
	ns['db']['profile']['bags'][playerID] = ns['db']['profile']['bags'][playerID] or {}
	ns['db']['profile']['bags'][playerID][bag] = scanBag(bag)
end
local function pout(msg)
	SELECTED_CHAT_FRAME:AddMessage(tostring(msg))
end

SlashCmdList[string.upper(name)] = function(msg)
	local um = string.upper(msg)
	for id, itemname in pairs(ns['db']['profile']['names']) do
		local un = string.upper(itemname)
		if(string.find(un,um)) then
			local itemname2,itemlink = GetItemInfo(id)
			local dout = {}
			if(id)then
				local res = getResult(id)
				if(not res) then
					res = bagLoop({},id,'count','bags')
					res = bagLoop(res,id,'bankCount','bank')
					setResult(id,res)
				end
				for name,b in pairs(res) do
					local c1 = b['count'] and b['count'] or 0
					local c2 = b['bankCount'] and b['bankCount'] or 0
					local line,sep = '',''
					if(c1 > 0) then
						sep = c2>0 and ',' or ''
						line = format(" Bags %s%s",c1,sep)
					end
					if(c2 > 0) then
						line = format("%s Bank %s",line,c2)
					end
					local dname, drealm = strsplit(" - ",name)
					dout[#dout+1] = (format("%s: %s [%s ] ",dname,c1 + c2, line))
				end
			end
			if(#dout>0) then
				pout(format("%s %s",getIcon(id),itemlink and itemlink or itemname))
				for k,v in pairs(dout) do
					pout(v)
				end
			end
		end
	end
end
_G["SLASH_"..string.upper(name).."1"] = "/find"
